package HTML::Prototype::Helper;

use strict;
use HTML::Prototype::Helper::Tag;

=head1 NAME

HTML::Prototype::Helper - Imports helper packages for HTML::Prototype

=head1 SYNOPSIS

	use HTML::Prototype::Helper;

=head1 SEE ALSO

L<HTML::Prototype>, L<http://prototype.conio.net/>

=head1 AUTHOR

Sascha Kiefer, C<esskar@cpan.org>

Built around Prototype by Sam Stephenson.
Much code is ported from Ruby on Rails javascript helpers.

=head1 LICENSE

This library is free software. You can redistribute it and/or modify it under
the same terms as perl itself.

=cut

1;
